# COPYIGHT 2001 DUMPLETON SOFTWARE CONSULTING PTY LIMITED

# Okay to import "SOAP.py" so long as this module
# is installed as "netsvc/soap/__init__.py". What
# will happen is that the top level "SOAP.py" will
# be imported and not this one from the subdirectory.
# This is a bit nasty but it seems to work okay. :-)
# Actually, this is only really an issue if Python
# ignores the case of files on Win32 platform.

import SOAP
import netsvc
import cgi
import string

class xsdType(SOAP.anyType):
  def __init__(self,data,name):
    SOAP.anyType.__init__(self,data,name)
    self._type = name

class xsd2Type(SOAP.anyType):
  _validURIs = (SOAP.NS.XSD2,)
  def __init__(self,data,name):
    SOAP.anyType.__init__(self,data,name)
    self._type = name

class xsd3Type(SOAP.anyType):
  _validURIs = (SOAP.NS.XSD3,)
  def __init__(self,data,name):
    SOAP.anyType.__init__(self,data,name)
    self._type = name

class RpcServlet(netsvc.HttpServlet,netsvc.Service):

  def __init__(self,session,binding):
    netsvc.HttpServlet.__init__(self,session)
    netsvc.Service.__init__(self,"","")
    self._binding = binding
    self._method = None
    self._content = []
    self._contentLength = 0

  def destroyServlet(self):
    self.destroyReferences()

  def decodeValue(self,name,string):
    # Builtin types already decoded by C++.
    # Binary special case as content decoded.
    if name == "xsd:base64Binary":
      return SOAP.binaryType(string)
    elif name == "xsd:dateTime":
      return xsdType(string,"timeInstant")
    elif name == "xsd:duration":
      return xsdType(string,"timeDuration")
    elif name[0:4] == "xsd:":
      return xsdType(string,name[4:])
    return netsvc.decodeValue(self,name,string)

  def encodeObject(self,object):
    # Boolean, date types etc undistinguishable.
    if isinstance(object,SOAP.arrayType):
      return (None,list(object))
    elif isinstance(object,SOAP.typedArrayType):
      return (None,list(object))
    elif isinstance(object,SOAP.structType):
      return (None,object._asdict)
    return netsvc.encodeObject(self,object)

  def processRequest(self):
    try:
      if self.requestMethod() != "POST":
	self.sendError(400,"Only POST method supported.")
	return
      if self.contentLength() <= 0:
	self.sendError(400,"Invalid content length.")
	return
    except:
      self.sendError(500,cgi.escape(netsvc.exceptionDetails()))

  def processContent(self,content):
    try:
      try:
        self._content.append(content)
	self._contentLength = self._contentLength + len(content)
	if self._contentLength < self.contentLength():
	  return
	if self._contentLength > self.contentLength():
	  error = netsvc.SERVER_REQUEST_DECODING_ERROR
	  description = netsvc.SERVER_REQUEST_DECODING_ERROR_MESSAGE
	  origin = "netsvc.soap"
	  details = "Invalid content length."
	  raise netsvc.ServiceFailure(error,description,origin,details)
	content = string.join(self._content,"")
        (request,header,body,attrs) = SOAP.parseSOAPRPC(
            content,header=1,body=1,attrs=1)
	self._method = request._name
	params = request._aslist
      except netsvc.ServiceFailure:
	raise
      except:
	error = netsvc.SERVER_REQUEST_DECODING_ERROR
	description = netsvc.SERVER_REQUEST_DECODING_ERROR_MESSAGE
	origin = "netsvc.soap"
	details = netsvc.exceptionDescription()
	raise netsvc.ServiceFailure(error,description,origin,details)

      service = self.serviceEndPoint(self._binding)
      id = apply(service.__getattr__(self._method),params)
      self.monitorResponse(self.processResult,id)

    except netsvc.ServiceFailure,exception:
      try:
	detail = {}
	detail["error"] = exception.error
	detail["description"] = exception.description
	detail["origin"] = exception.origin
	detail["details"] = exception.details
	body = SOAP.buildSOAP(SOAP.faultType(
	    "%s:Client"%SOAP.NS.ENV_T,"Request Failed",detail))
	self.sendResult(500,body)
      except:
	self.sendError(500,cgi.escape(netsvc.exceptionDetails()))
    except:
      self.sendError(500,cgi.escape(netsvc.exceptionDetails()))

  def processResult(self,params):
    try:
      if self.failureOccurred():
	exeception = self.currentFailure()
        error = exeception.error()
        description = exeception.description()
        origin = exeception.origin()
        details = exeception.details()
	raise netsvc.ServiceFailure(error,description,origin,details)
      body = SOAP.buildSOAP(kw={'%sResponse'%self._method:params})
      self.sendResult(200,body)
    except netsvc.ServiceFailure,exception:
      try:
	detail = {}
	detail["error"] = exception.error
	detail["description"] = exception.description
	detail["origin"] = exception.origin
	detail["details"] = exception.details
	if exception.origin == "netsvc" and \
	    (exception.error == netsvc.SERVER_METHOD_UNAVAILABLE or \
	    exception.error == netsvc.SERVER_PARAMETERS_INCORRECT or \
	    exception.error == netsvc.SERVER_REQUEST_DECODING_ERROR):
	  body = SOAP.buildSOAP(SOAP.faultType(
	      "%s:Client"%SOAP.NS.ENV_T,"Request Failed",detail))
	else:
	  body = SOAP.buildSOAP(SOAP.faultType(
	      "%s:Server"%SOAP.NS.ENV_T,"Request Failed",detail))
	self.sendResult(500,body)
      except:
	self.sendError(500,cgi.escape(netsvc.exceptionDetails()))
    except:
      self.sendError(500,cgi.escape(netsvc.exceptionDetails()))

  def sendResult(self,status,body):
    self.sendResponse(status)
    self.sendHeader("Content-Type","text/xml")
    self.sendHeader("Content-Length",str(len(body)))
    self.endHeaders()
    self.sendContent(body)
    self.endContent()


class RpcGateway(netsvc.HttpServer,netsvc.Monitor):

  def __init__(self,group):
    netsvc.HttpServer.__init__(self)
    netsvc.Monitor.__init__(self)
    self._members = {}
    self._bindings = {}
    self.subscribeServiceGroup(self.register,group)

  def register(self,binding,group,status):
    serviceName = binding.serviceName()
    agentIdentity = binding.agentIdentity()
    if status == netsvc.SERVICE_AVAILABLE:
      if not self._bindings.has_key(agentIdentity):
	self._bindings[agentIdentity] = binding
	if self._members.has_key(serviceName):
	  self._members[serviceName].append(agentIdentity)
	else:
	  self._members[serviceName] = [agentIdentity]
    else:
      if self._bindings.has_key(agentIdentity):
        del self._bindings[agentIdentity]
	members = self._members[serviceName]
	del members[members.index(agentIdentity)]
	if len(members) == 0:
	  del self._members[serviceName]

  def servlet(self,serverRoot,servletPath,session):
    if self._members.has_key(servletPath):
      agentIdentity = self._members[servletPath][0]
      binding = self._bindings[agentIdentity]
      return RpcServlet(session,binding)
    return None
